# Generated by Django 3.2.19 on 2023-06-22 14:36

from django.db import migrations

from core.models import Role, RoleRight
from core.utils import insert_role_right_for_system

TASK_TRIAGE_ID = 2097152
IMIS_ADMIN = 64
ROLE_NAME = "Task Triage"
TASK_TRIAGE_ROLE_RIGHTS = ["190001", "190002", "190003", "190004"]


def _get_role(role_id):
    return Role.objects.filter(is_system=role_id).first()


def _add_rights_to_role(role):
    for right in TASK_TRIAGE_ROLE_RIGHTS:
        insert_role_right_for_system(role, right)


def _remove_rights_from_role(role):
    RoleRight.objects.filter(
        role__is_system=role,
        right_id__in=TASK_TRIAGE_ROLE_RIGHTS,
        validity_to__isnull=True
    ).delete()


def _create_task_triage_role():
    role = _get_role(TASK_TRIAGE_ID)
    if not role:
        task_triage = Role(is_system=TASK_TRIAGE_ID, name=ROLE_NAME, is_blocked=False)
        task_triage.save()


def _delete_task_triage_role():
    role = _get_role(TASK_TRIAGE_ID)
    if role:
        role.delete()


def on_migration(apps, schema_editor):
    _create_task_triage_role()
    _add_rights_to_role(IMIS_ADMIN)
    _add_rights_to_role(TASK_TRIAGE_ID)


def on_migration_reverse(apps, schema_editor):
    _remove_rights_from_role(IMIS_ADMIN)
    _remove_rights_from_role(TASK_TRIAGE_ID)
    _delete_task_triage_role()


class Migration(migrations.Migration):
    dependencies = [
        ('tasks_management', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(on_migration, on_migration_reverse)
    ]
